@@ -1,14 +1,17 @@
 # This file is part of Mach-O Loader for CLE.
 # Contributed December 2016 by Fraunhofer SIT (https://www.sit.fraunhofer.de/en/) and updated in September 2019.
+from __future__ import annotations
 
 import logging
 import struct
-from typing import TYPE_CHECKING, Callable, Dict, Tuple
+from collections.abc import Callable
+from typing import TYPE_CHECKING
 
 from cle.address_translator import AT
 from cle.backends.relocation import Relocation
 from cle.errors import CLEInvalidBinaryError
 
+from .macho_enums import RebaseOpcode, RebaseType
 from .symbol import AbstractMachOSymbol, BindingSymbol, DyldBoundSymbol, SymbolTableSymbol
 
 if TYPE_CHECKING:
@@ -44,7 +47,7 @@ else:
     chh = ord
 
 
-def read_uleb(blob: bytes, offset: int) -> Tuple[int, int]:
+def read_uleb(blob: bytes, offset: int) -> tuple[int, int]:
     """Reads a number encoded as uleb128"""
     result = 0
     shift = 0
@@ -120,7 +123,7 @@ class BindingHelper:
     """Factors out binding logic from MachO.
     Intended to work in close conjunction with MachO not for standalone use"""
 
-    binary: "MachO"
+    binary: MachO
 
     def __init__(self, binary):
         self.binary = binary
@@ -201,11 +204,108 @@ class BindingHelper:
 
         log.debug("Done binding lazy symbols")
 
+    def do_rebases(self, blob: bytes):
+        """
+        Handles the rebase blob
+        Implementation based closely on ImageLoaderMachOCompressed::rebase from dyld
+        https://github.com/apple-opensource/dyld/blob/e3f88907bebb8421f50f0943595f6874de70ebe0/src/ImageLoaderMachOCompressed.cpp#L382-L463
+
+        :param blob:
+        :return:
+        """
+        if blob is None:
+            return
+
+        # State variables
+        reloc_type: RebaseType | None = None
+        done = False
+        segment = None
+        address = None
+        index = 0
+        end = len(blob)
+        while not done and index < end:
+            opcode, immediate = RebaseOpcode.parse_byte(blob[index])
+            index += 1
+
+            if opcode == RebaseOpcode.DONE:
+                done = True
+
+            elif opcode == RebaseOpcode.SET_TYPE_IMM:
+                reloc_type = RebaseType(immediate)
+
+            elif opcode == RebaseOpcode.SET_SEGMENT_AND_OFFSET_ULEB:
+                segment = self.binary.segments[immediate]
+                offset, index = self.read_uleb(blob, index)
+                address = segment.vaddr + offset
+
+            elif opcode == RebaseOpcode.ADD_ADDR_ULEB:
+                uleb, index = self.read_uleb(blob, index)
+                address += uleb
+
+            elif opcode == RebaseOpcode.ADD_ADDR_IMM_SCALED:
+                address += immediate * self.binary.arch.bytes
+
+            elif opcode == RebaseOpcode.DO_REBASE_IMM_TIMES:
+                for _ in range(immediate):
+                    self.rebase_at(address, reloc_type)
+                    address += self.binary.arch.bytes
+
+            elif opcode == RebaseOpcode.DO_REBASE_ULEB_TIMES:
+                count, index = self.read_uleb(blob, index)
+                for _ in range(count):
+                    if address >= segment.vaddr + segment.memsize:
+                        raise CLEInvalidBinaryError()
+                    self.rebase_at(address, reloc_type)
+                    address += self.binary.arch.bytes
+
+            elif opcode == RebaseOpcode.DO_REBASE_ADD_ADDR_ULEB:
+                self.rebase_at(address, reloc_type)
+                uleb, index = self.read_uleb(blob, index)
+                address += uleb + self.binary.arch.bytes
+
+            elif opcode == RebaseOpcode.DO_REBASE_ULEB_TIMES_SKIPPING_ULEB:
+                count, index = self.read_uleb(blob, index)
+                skip, index = self.read_uleb(blob, index)
+                for _ in range(count):
+                    if address >= segment.vaddr + segment.memsize:
+                        raise CLEInvalidBinaryError()
+                    self.rebase_at(address, reloc_type)
+                    address += skip + self.binary.arch.bytes
+
+            else:
+                raise CLEInvalidBinaryError(f"Invalid opcode for current binding: {opcode:#x}")
+
+    @staticmethod
+    def read_uleb(blob, offset) -> tuple[int, int]:
+        """
+        little helper to read ulebs, that also returns the new index
+        :param blob:
+        :param offset:
+        :return:
+        """
+        uleb, length = read_uleb(blob, offset)
+        return uleb, offset + length
+
+    def rebase_at(self, address: int, ty: RebaseType):
+        relative_rebase_location = AT.from_lva(address, self.binary).to_rva()
+        unslid_pointer = self.binary.memory.unpack_word(relative_rebase_location)
+        relative_pointer = AT.from_lva(unslid_pointer, self.binary).to_rva()
+
+        if ty == RebaseType.POINTER:
+            reloc = MachOPointerRelocation(self.binary, relative_rebase_location, relative_pointer)
+        elif ty == RebaseType.TEXT_ABSOLUTE32:
+            reloc = MachOPointerRelocation(self.binary, relative_rebase_location, relative_pointer)
+        elif ty == RebaseType.TEXT_PCREL32:
+            raise NotImplementedError()
+        else:
+            raise ValueError(f"Invalid rebase type: {ty:#x}")
+        self.binary.relocs.append(reloc)
+
     def _do_bind_generic(
         self,
         blob,
         init_state: BindingState,
-        opcode_dict: Dict[int, Callable[[BindingState, "MachO", int, bytes], BindingState]],
+        opcode_dict: dict[int, Callable[[BindingState, MachO, int, bytes], BindingState]],
     ):
         """
         Does the actual binding work. Represents a generic framework for interpreting binding opcodes
@@ -236,19 +336,19 @@ class BindingHelper:
 # pylint: disable=unused-argument
 # The following functions realize different variants of handling binding opcodes
 # the format is def X(state,binary,immediate,blob) => state
-def n_opcode_done(s: BindingState, _b: "MachO", _i: int, _blob: bytes) -> BindingState:
+def n_opcode_done(s: BindingState, _b: MachO, _i: int, _blob: bytes) -> BindingState:
     log.debug("BIND_OPCODE_DONE @ %#x", s.index)
     s.done = True
     return s
 
 
-def n_opcode_set_dylib_ordinal_imm(s: BindingState, _b: "MachO", i: int, _blob: bytes) -> BindingState:
+def n_opcode_set_dylib_ordinal_imm(s: BindingState, _b: MachO, i: int, _blob: bytes) -> BindingState:
     log.debug("SET_DYLIB_ORDINAL_IMM @ %#x: %d", s.index, i)
     s.lib_ord = i
     return s
 
 
-def n_opcode_set_dylib_ordinal_uleb(s: BindingState, _b: "MachO", _i: int, blob: bytes) -> BindingState:
+def n_opcode_set_dylib_ordinal_uleb(s: BindingState, _b: MachO, _i: int, blob: bytes) -> BindingState:
     uleb = read_uleb(blob, s.index)
     s.lib_ord = uleb[0]
     s.index += uleb[1]
@@ -256,7 +356,7 @@ def n_opcode_set_dylib_ordinal_uleb(s: BindingState, _b: "MachO", _i: int, blob:
     return s
 
 
-def n_opcode_set_dylib_special_imm(s: BindingState, _b: "MachO", i: int, _blob: bytes) -> BindingState:
+def n_opcode_set_dylib_special_imm(s: BindingState, _b: MachO, i: int, _blob: bytes) -> BindingState:
     if i == 0:
         s.lib_ord = 0
     else:
@@ -265,7 +365,7 @@ def n_opcode_set_dylib_special_imm(s: BindingState, _b: "MachO", i: int, _blob:
     return s
 
 
-def n_opcode_set_trailing_flags_imm(s: BindingState, _b: "MachO", i: int, blob: bytes) -> BindingState:
+def n_opcode_set_trailing_flags_imm(s: BindingState, _b: MachO, i: int, blob: bytes) -> BindingState:
     s.sym_name = ""
     s.sym_flags = i
 
@@ -278,14 +378,14 @@ def n_opcode_set_trailing_flags_imm(s: BindingState, _b: "MachO", i: int, blob:
     return s
 
 
-def n_opcode_set_type_imm(s: BindingState, _b: "MachO", i: int, _blob: bytes) -> BindingState:
+def n_opcode_set_type_imm(s: BindingState, _b: MachO, i: int, _blob: bytes) -> BindingState:
     # pylint: disable=unused-argument
     s.binding_type = i
     log.debug("SET_TYPE_IMM @ %#x: %d", s.index, s.binding_type)
     return s
 
 
-def n_opcode_set_addend_sleb(s: BindingState, _b: "MachO", _i: int, blob: bytes) -> BindingState:
+def n_opcode_set_addend_sleb(s: BindingState, _b: MachO, _i: int, blob: bytes) -> BindingState:
     sleb = read_sleb(blob, s.index)
     s.addend = sleb[0]
     log.debug("SET_ADDEND_SLEB @ %#x: %d", s.index, s.addend)
@@ -293,7 +393,7 @@ def n_opcode_set_addend_sleb(s: BindingState, _b: "MachO", _i: int, blob: bytes)
     return s
 
 
-def n_opcode_set_segment_and_offset_uleb(s: BindingState, b: "MachO", i: int, blob: bytes) -> BindingState:
+def n_opcode_set_segment_and_offset_uleb(s: BindingState, b: MachO, i: int, blob: bytes) -> BindingState:
     s.segment_index = i
     uleb = read_uleb(blob, s.index)
     log.debug("(n)SET_SEGMENT_AND_OFFSET_ULEB @ %#x: %d, %d", s.index, s.segment_index, uleb[0])
@@ -305,7 +405,7 @@ def n_opcode_set_segment_and_offset_uleb(s: BindingState, b: "MachO", i: int, bl
     return s
 
 
-def l_opcode_set_segment_and_offset_uleb(s: BindingState, b: "MachO", i: int, blob: bytes) -> BindingState:
+def l_opcode_set_segment_and_offset_uleb(s: BindingState, b: MachO, i: int, blob: bytes) -> BindingState:
     uleb = read_uleb(blob, s.index)
     log.debug("(l)SET_SEGMENT_AND_OFFSET_ULEB @ %#x: %d, %d", s.index, i, uleb[0])
     seg = b.segments[i]
@@ -314,7 +414,7 @@ def l_opcode_set_segment_and_offset_uleb(s: BindingState, b: "MachO", i: int, bl
     return s
 
 
-def n_opcode_add_addr_uleb(s: BindingState, _b: "MachO", _i: int, blob: bytes) -> BindingState:
+def n_opcode_add_addr_uleb(s: BindingState, _b: MachO, _i: int, blob: bytes) -> BindingState:
     uleb = read_uleb(blob, s.index)
     s.add_address_ov(s.address, uleb[0])
     log.debug("ADD_ADDR_ULEB @ %#x: %d", s.index, uleb[0])
@@ -322,7 +422,7 @@ def n_opcode_add_addr_uleb(s: BindingState, _b: "MachO", _i: int, blob: bytes) -
     return s
 
 
-def n_opcode_do_bind(s: BindingState, b: "MachO", _i: int, _blob: bytes) -> BindingState:
+def n_opcode_do_bind(s: BindingState, b: MachO, _i: int, _blob: bytes) -> BindingState:
     log.debug("(n)DO_BIND @ %#x", s.index)
     s.check_address_bounds()
     s.bind_handler(s, b)
@@ -330,13 +430,13 @@ def n_opcode_do_bind(s: BindingState, b: "MachO", _i: int, _blob: bytes) -> Bind
     return s
 
 
-def l_opcode_do_bind(s: BindingState, b: "MachO", _i: int, _blob: bytes) -> BindingState:
+def l_opcode_do_bind(s: BindingState, b: MachO, _i: int, _blob: bytes) -> BindingState:
     log.debug("(l)DO_BIND @ %#x", s.index)
     s.bind_handler(s, b)
     return s
 
 
-def n_opcode_do_bind_add_addr_uleb(s: BindingState, b: "MachO", _i: int, blob: bytes) -> BindingState:
+def n_opcode_do_bind_add_addr_uleb(s: BindingState, b: MachO, _i: int, blob: bytes) -> BindingState:
     uleb = read_uleb(blob, s.index)
     log.debug("DO_BIND_ADD_ADDR_ULEB @ %#x: %d", s.index, uleb[0])
     if s.address >= s.seg_end_address:
@@ -351,7 +451,7 @@ def n_opcode_do_bind_add_addr_uleb(s: BindingState, b: "MachO", _i: int, blob: b
     return s
 
 
-def n_opcode_do_bind_add_addr_imm_scaled(s: BindingState, b: "MachO", i: int, _blob: bytes) -> BindingState:
+def n_opcode_do_bind_add_addr_imm_scaled(s: BindingState, b: MachO, i: int, _blob: bytes) -> BindingState:
     log.debug("DO_BIND_ADD_ADDR_IMM_SCALED @ %#x: %d", s.index, i)
     if s.address >= s.seg_end_address:
         log.error(
@@ -367,7 +467,7 @@ def n_opcode_do_bind_add_addr_imm_scaled(s: BindingState, b: "MachO", i: int, _b
     return s
 
 
-def n_opcode_do_bind_uleb_times_skipping_uleb(s: BindingState, b: "MachO", _i: int, blob: bytes) -> BindingState:
+def n_opcode_do_bind_uleb_times_skipping_uleb(s: BindingState, b: MachO, _i: int, blob: bytes) -> BindingState:
     count = read_uleb(blob, s.index)
     s.index += count[1]
     skip = read_uleb(blob, s.index)
@@ -387,12 +487,12 @@ def n_opcode_do_bind_uleb_times_skipping_uleb(s: BindingState, b: "MachO", _i: i
     return s
 
 
-class MachORelocation(Relocation):
+class MachOSymbolRelocation(Relocation):
     """
     Generic Relocation for MachO. It handles relocations that point to symbols
     """
 
-    def __init__(self, owner: "MachO", symbol: AbstractMachOSymbol, relative_addr: int, data):
+    def __init__(self, owner: MachO, symbol: AbstractMachOSymbol, relative_addr: int, data):
         super().__init__(owner, symbol, relative_addr)
         self.data = data
 
@@ -423,14 +523,13 @@ class MachORelocation(Relocation):
         return f"<MachO Reloc for {self.symbol} at {hex(self.relative_addr)}>"
 
 
-class MachOChainedFixup(Relocation):
+class MachOPointerRelocation(Relocation):
     """
-    A special kind of relocation that handles internal pointers in the binary.
-    This was introduced with iOS15+ and is somewhat explained here
-    https://github.com/datatheorem/strongarm/blob/release/chained_fixup_pointers.md
+    A relocation for a pointer without any associated symbol
+    These are either generated while handling the rebase blob, or while parsing chained fixups
     """
 
-    def __init__(self, owner: "MachO", relative_addr: int, data):
+    def __init__(self, owner: MachO, relative_addr: int, data):
         """
 
         :param owner:
@@ -461,7 +560,7 @@ class MachOChainedFixup(Relocation):
 
 
 # default binding handler
-def default_binding_handler(state: BindingState, binary: "MachO"):
+def default_binding_handler(state: BindingState, binary: MachO):
     """Binds location to the symbol with the given name and library ordinal"""
 
     # locate the symbol:
@@ -487,7 +586,7 @@ def default_binding_handler(state: BindingState, binary: "MachO"):
         log.debug("Updating address %#x with symobl %r @ %#x", location, state.sym_name, value)
         addr = AT.from_lva(location, binary).to_rva()
         data = struct.pack(binary.struct_byteorder + ("Q" if binary.arch.bits == 64 else "I"), value)
-        reloc = MachORelocation(binary, symbol, addr, data)
+        reloc = MachOSymbolRelocation(binary, symbol, addr, data)
         binary.relocs.append(reloc)
         symbol.bind_xrefs.append(location)
     elif state.binding_type == 2:  # ABSOLUTE32
