@@ -1,12 +1,14 @@
+from __future__ import annotations
+
 import logging
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 from cle.address_translator import AT
 
 from .symbol import Symbol, SymbolType
 
 if TYPE_CHECKING:
-    from typing import Any, List
+    from typing import Any
 
     from .backend import Backend
 
@@ -28,12 +30,12 @@ class Relocation:
     :ivar resolved:         Whether the application of this relocation was successful
     """
 
-    def __init__(self, owner: "Backend", symbol: Optional[Symbol], relative_addr: int):
+    def __init__(self, owner: Backend, symbol: Symbol | None, relative_addr: int):
         self.owner = owner
         self.arch = owner.arch
         self.symbol = symbol
         self.relative_addr = relative_addr
-        self.resolvedby: Optional[Symbol] = None
+        self.resolvedby: Symbol | None = None
         self.resolved: bool = False
         self.resolvewith = None
         if self.symbol is not None and self.symbol.is_import:
@@ -42,7 +44,7 @@ class Relocation:
     AUTO_HANDLE_NONE = False
 
     def resolve_symbol(
-        self, solist: "List[Any]", thumb=False, extern_object=None, **kwargs
+        self, solist: list[Any], thumb=False, extern_object=None, **kwargs
     ):  # pylint: disable=unused-argument
         if self.resolved:
             return
@@ -84,7 +86,7 @@ class Relocation:
         new_symbol = extern_object.make_extern(self.symbol.name, sym_type=self.symbol._type, thumb=thumb)
         self.resolve(new_symbol, extern_object=extern_object)
 
-    def resolve(self, obj, **kwargs):  # pylint: disable=unused-argument
+    def resolve(self, obj, extern_object=None):  # pylint: disable=unused-argument
         self.resolvedby = obj
         self.resolved = True
         if self.symbol is not None:
